/*
// File:		Imap.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2006 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include <iostream>
#include <EmaiIMAP.hpp>
#include <EmaiMessage.hpp>
#include "ImapFolder.hpp"
#include "ImapResponseHandler.hpp"
#include "StreamUtils.hpp"
#include "ImapMessageData.hpp"
#include "MessageComposer.hpp"
#include "MessagePartEnumerator.hpp"
#include "ImapCallback.hpp"
#include "PrintMessage.hpp"

/*IMAP account settings, replace them with real account settings*/
EmaiUint32 sTimeout							= 60;
EmaiUniChar sMailServer[]					= {'m', 'a', 'i', 'l', '.', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};
EmaiUniChar sMailServerAccount[]			= {'m', 'y', 'a', 'c', 'c', 'o', 'u', 'n', 't', '\0'};
EmaiUniChar sMailServerPassword[]			= {'m', 'y', 'p', 'a', 's', 's', 'w', 'o', 'r', 'd', '\0'};
EmaiOptions sAuthType						= EmaiIMAPAuthAutomatic;

/*Names of the folders*/
EmaiUniChar sCreatingDeletingFolderName[]	= {'T', 'e', 'm', 'p', ' ', 'f', 'o', 'l', 'd', 'e', 'r', '\0'};
EmaiUniChar sAddingMessageFolderName[]		= {'i', 'n', 'b', 'o', 'x', '\0'};
EmaiUniChar sDownloadingMessageFolderName[]	= {'i', 'n', 'b', 'o', 'x', '\0'};
EmaiUniChar sDeletingMessageFolderName[]	= {'i', 'n', 'b', 'o', 'x', '\0'};
EmaiUniChar sSearchingMessageFolderName[]	= {'i', 'n', 'b', 'o', 'x', '\0'};
EmaiUniChar sSubsribingMessageFolderName[]	= {'T', 'e', 'm', 'p', ' ', 'f', 'o', 'l', 'd', 'e', 'r', '\0'};

/*Contact fields of the message*/
EmaiUniChar sContactToName[]				= {'T', 'o', 'm', '\0'};
EmaiUniChar sContactToAddress[]				= {'t', 'o', 'm', '@', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm', '\0'};

EmaiUniChar sContactFromName[]				= {'N', 'i', 'c', 'k', '\0'};
EmaiUniChar sContactFromAddress[]			= {'n', 'i', 'c', 'k', '@', 'm', 'y', 'd', 'o', 'm', 'a', 'i', 'n', '.', 'c', 'o', 'm'};


EmaiUniChar sMessageSubject[]				= {'N', 'e', 'w', ' ', 'b', 'o', 'o', 'k', '\0'};
EmaiConstCharPtr sPlainTextMessage			= "Hi Tom.\n\nI yesterday have bought the new book, hope, that it should be interesting.\n\nGood-bye!";


/*Constants*/
EmaiUniChar sEmptyString[]					= {'\0'};
EmaiUniChar sWildCardString[]				= {'*', '\0'};


/*Create or delete the folder, depending of folder presence or absense, accordingly*/
void
CreateDeleteFolder()
{
	std::cout << "Creating or deleting of the folder has been started!" << std::endl;

	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);

		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);
		
		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);

		/*Check if the folder is absent in the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sCreatingDeletingFolderName);

		if (folder == NULL)
		{
			/*Create the folder*/
			imapSession.CreateMailbox(sCreatingDeletingFolderName);

			std::cout << "Your folder message successfully has been created." <<
								std::endl << std::endl;
		}
		else
		{
			/*Delete the folder*/
			imapSession.DeleteMailbox(sCreatingDeletingFolderName);

			std::cout << "Your folder message successfully has been deleted." <<
								std::endl << std::endl;
		}
		
		/*Disconnect from the server*/
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Error number " << exception.GetErrorCode() << "." <<
								std::endl << std::endl;
	}

	std::cout << "Creating or deleting of the folder has been finished!" <<
								std::endl << std::endl;
}

/*Add the message in the folder*/
void
AddMessage()
{
	std::cout << "Addition of the message has been started!" << std::endl;
	
	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);

		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);

		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);
		
		/*Check if the folder is present at the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sAddingMessageFolderName);

		if (folder != NULL)
		{
			/*Compose the message*/

			/*Create the message composer*/
			MessageComposer messageComposer;

			/*Add the message text part*/
			messageComposer.SetText(sPlainTextMessage, EmaiUSASCII);

			/*Add contact fields to the message*/
			Emai::ContactList contactTo;
			contactTo.AddContact(sContactToName, sContactToAddress);
			Emai::ContactList contactFrom;
			contactFrom.AddContact(sContactFromName, sContactFromAddress);

			Emai::Message message = messageComposer.ComposeMessage(contactFrom, NULL, contactTo,
									NULL, NULL, sMessageSubject);
			
			if (message != NULL)
			{
				/*Add the message to the folder*/
				imapSession.AppendMessage(sAddingMessageFolderName, NULL, NULL, message);
			}

			std::cout << "Your message successfully has been added." << std::endl << std::endl;
		}
		else
		{
			std::cout << "There isn't specified folder in mail account." << std::endl << std::endl;
		}
		
		/*Disconnect from the server*/
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Error number " << exception.GetErrorCode() << "." << std::endl;
	}

	std::cout << "Addition of the message has been finished!" << std::endl << std::endl;
}

/*Download the message from the folder*/
void
DownloadMessages()
{
	std::cout << "Downloading of the message has been started!" << std::endl;
	
	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);
		
		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);
		
		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);
		
		/*Check if the folder is present at the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sDownloadingMessageFolderName);
		
		if (folder != NULL)
		{
			/*Select the folder*/
			imapSession.SelectMailbox(sDownloadingMessageFolderName);

			EmaiUint32 messageNumber = folder->GetExists();
			std::cout << "Number of the existent files: " << messageNumber << std::endl;

			/*Fetch all messages from the folder*/
			
			EmaiFetchMessageAttribute fetchAttr = {EmaiFetchRFC822};
			EmaiFetchMessageData fetchData = {EmaiStructureVersionFirst};
			fetchData.selector = EmaiDataFetchAttr;
			fetchData.param.attr.attributeCount = 1;
			fetchData.param.attr.attributes = &fetchAttr;

			for (EmaiUint32 i = 1; i <= messageNumber; ++i)
			{
				Emai::ImapSequenceSet msgSequence;
				msgSequence.AddNumber(i);
				
				/*Fetch the message with number "i"*/
				imapSession.Fetch(msgSequence, &fetchData);
			
				Emai::Message message = folder->GetMessageDataAtIndex(i)->GetMessage();

				/*Create the message enumerator*/
				MessagePartEnumerator messagePartEnumerator(message);
				messagePartEnumerator.Enumerate();

				PrintMessage(messagePartEnumerator);
			}
			
			std::cout << "Your messages successfully have been downloaded." << std::endl << std::endl;
		}
		else
		{
			std::cout << "There isn't specified folder in mail account." << std::endl << std::endl;
		}
		
		/*Disconnect from the server*/
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the number of error*/
		std::cout << "Error number " << exception.GetErrorCode() << "." << std:: endl << std::endl;
	}

	std::cout << "Downloading of the message has been finished!" << std::endl << std::endl;
}

/*Search the selected folder for the messages with certain criterias*/
void
SearchMessages()
{
	std::cout << "Searching of the messages has been started!" << std::endl;

	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);

		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);

		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);
		
		/*Check if the folder is present at the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sSearchingMessageFolderName);
		
		if (folder != NULL)
		{
			imapSession.SelectMailbox(sDownloadingMessageFolderName);
			
			/*Create the criterias for the messages search*/
			EmaiSearchingCriteria searchCriteria;
			
			searchCriteria.version = EmaiStructureVersionFirst;
			searchCriteria.characterSet = EmaiUnknown;
			
			searchCriteria.selector = EmaiSearchingCriteriaType;
			
			const EmaiUniChar criteria[] = {'T', 'E', 'X', 'T', ' ', 'b', 'o', 'o', 'k', '\0'};
			searchCriteria.data.searchingCriteria = criteria;
			
			/*Search the messages*/
			imapSession.Search(&searchCriteria);
			
			EmaiFetchMessageAttribute fetchAttr = {EmaiFetchRFC822};
			EmaiFetchMessageData fetchData = {EmaiStructureVersionFirst};
			fetchData.selector = EmaiDataFetchAttr;
			fetchData.param.attr.attributeCount = 1;
			fetchData.param.attr.attributes = &fetchAttr;

			/*Create the list of the found messages*/
			Emai::ImapSequenceSet foundMessageList = folder->GetFoundMessageList();

			std::cout << "number of found messages: " << foundMessageList.GetCount() << std::endl;

			/*Process all messages in the list*/
			EmaiUint32 messageNumber = 0;
			foundMessageList.GetFirstNumber(messageNumber);
			do
			{
				std::cout << "\tmessage " << messageNumber << std::endl << std::endl;
				
				Emai::ImapSequenceSet msgSequence;
				msgSequence.AddNumber(messageNumber);
				imapSession.Fetch(msgSequence, &fetchData);
				
				Emai::Message message = folder->GetMessageDataAtIndex(messageNumber)->GetMessage();
				
				/*Create the message enumerator*/
				MessagePartEnumerator messagePartEnumerator(message);
				
				/*Enumerate the message parts*/
				messagePartEnumerator.Enumerate();
				
				PrintMessage(messagePartEnumerator);

			} while (foundMessageList.GetNextNumberForward(messageNumber, messageNumber));
		}
		else
		{
			std::cout << "There isn't specified folder in mail account." << std::endl << std::endl;
		}

		/*Disconnect from the server*/
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Error number " << exception.GetErrorCode() << "." << std::endl << std::endl;
	}

	std::cout << "Searching of the messages has been finished!" << std::endl << std::endl;
}

/*Delete all messages in the selected folder*/
void
DeleteMessages()
{
	std::cout << "Deleting of the message has been started!" << std::endl;
	
	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);

		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);

		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);
		
		/*Check if the folder is present at the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sDeletingMessageFolderName);

		if (folder != NULL)
		{
			imapSession.SelectMailbox(sDownloadingMessageFolderName);

			EmaiUint32 messageNumber = folder->GetExists();

			EmaiStoreMessageData storeData = {EmaiStructureVersionFirst};
			storeData.selector = EmaiDataStoreAttr;
			storeData.param.attr.options = EmaiStoreAddFlags;
			storeData.param.attr.flags.flags = EmaiDeletedFlag;
			storeData.param.attr.flags.customFlags = NULL;

			/*Process all messsages in the folder*/
//			for (EmaiUint32 i = 1; i <= messageNumber; ++i)
			{
				Emai::ImapSequenceSet msgSequence;
				msgSequence.AddNumber(messageNumber);//(i);

				/*Select the current message*/
				imapSession.Store(msgSequence, &storeData);
				
				std::cout << "Message marked as deleted: " << messageNumber << std::endl << std::endl;
			}

			/*Expunge all messages, which have been selected*/
			Emai::ImapSequenceSet msgSequence;
//			msgSequence.AddRange(1, messageNumber);
			msgSequence.AddNumber(messageNumber);
			imapSession.Expunge(msgSequence);
			
			std::cout << "Your messages successfully have been deleted." << std::endl << std::endl;
		}
		else
		{
			std::cout << "There isn't specified folder in mail account." << std::endl << std::endl;
		}
		
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Error number " << exception.GetErrorCode() << "." << std::endl << std::endl;
	}

	std::cout << "Deleting of the messages has been finished!" << std::endl << std::endl;
}


/*Subcribe/unsubcribe the selected folder*/
void
SubscribeUnsubscribeFolder()
{
	std::cout << "Subscribing/unsubscribing of the folder has been started!" << std::endl;
	
	try
	{
		/*Create the session*/
		ImapCallback statusCallback;
		ImapResponseHandler imapResponseHandler;
		Emai::ImapSession imapSession(&imapResponseHandler, statusCallback, sTimeout);

		/*Connect to the server*/
		imapSession.Connect(sMailServer);

		/*Authenticate*/
		imapSession.Authenticate(sMailServerAccount, sMailServerPassword, sAuthType);

		/*Create the list of the folders*/
		imapSession.ListMailboxes(sEmptyString, sWildCardString);

		/*Check if the folder is present at the list*/
		ImapFolder* folder = imapResponseHandler.FindFolderByName(sSubsribingMessageFolderName);

		if (folder != NULL)
		{
			/*Create the list of the subscribed folders*/
			imapSession.ListSubscribedMailboxes(sEmptyString, sWildCardString);

			/*Find the selected folder in the list*/
			folder = imapResponseHandler.FindSubscribedFolderByName(sSubsribingMessageFolderName);

			if (folder != NULL)
			{
				/*Unsubscribe from the selected folder*/
				imapSession.UnsubscribeMailbox(sSubsribingMessageFolderName);
				std::cout << "Message folder has been unsubscribed!" << std::endl << std::endl;
			}
			else
			{
				/*Subscribe to the selected folder*/
				imapSession.SubscribeMailbox(sSubsribingMessageFolderName);
				std::cout << "Message folder has been subscribed!" << std::endl << std::endl;
			}
		}
		else
		{
			std::cout << "There isn't specified folder in mail account." << std::endl << std::endl;
		}
		
		/*Disconnect from the server*/
		imapSession.Disconnect();
	}
	catch (Emai::Exception& exception)
	{
		/*Show the error code*/
		std::cout << "Error number " << exception.GetErrorCode() << "." << std::endl << std::endl;
	}

	std::cout << "Subscribing/ubsubscribing of the folder has been finished!" << std::endl << std::endl;
}

int
main()
{
	CreateDeleteFolder();
	AddMessage();
	DownloadMessages();
	SearchMessages();
	DeleteMessages();

	SubscribeUnsubscribeFolder();
	SubscribeUnsubscribeFolder();

	std::cout << "Done! Press <enter>." << std::endl;
	std::cin.get();

	return 0;
}
