/*
// File:		ImapCallback.cpp
//
// Version:		mail.it library 1.0
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "ImapCallback.hpp"
#include "StreamUtils.hpp"

EmaiError
ImapCallback::OnOperation(
		Emai::ImapSession& inSession,
		EmaiUint32 inOperationCode
	)
{
	EmaiUint32 statSize = sizeof(NSTextField*);
	NSTextField* status = NULL;
	inSession.GetUserData(ImapStatusText, &status, statSize);
	NSString* statusTexts[] = {
		@"Connecting to server...",
		@"Successfully connected to server.",
		@"Authenticating on server...",
		@"Successfully authenticated on server.",
		@"Creating folder...",
		@"Folder successfully created.",
		@"Deleting folder...",
		@"Folder successfully deleted.",
		@"Subsribing folder...",
		@"Folder successfully subscribed.",
		@"Unsubsribing folder...",
		@"Folder successfully unsubscribed.",
		@"Renaming folder...",
		@"Folder successfully renamed.",
		@"Retrieving folder list...",
		@"Folder list successfully retrieved.",
		@"Examining folder...",
		@"Folder successfully examined.",
		@"Retrieving folder status...",
		@"Folder status successfully retrieved.",
		@"Selecting folder...",
		@"Folder successfully selected.",
		@"Closing folder...",
		@"Folder successfully closed.",
		@"Checking...",
		@"Checked.",
		@"Appending message...",
		@"Message successfully appended.",
		@"Searching messages...",
		@"Message search finished.",
		@"Fetching messages...",
		@"Messages successfully fetched.",
		@"Copying messages...",
		@"Messages successfully copied.",
		@"Storing messages data...",
		@"Messages data successfully stored.",
		@"Expunging messages...",
		@"Messages successfully expunged.",
		@"Sending NOOP.",
		@"NOOP successfully sent.",
		@"Closing connection.",
		@"Connection successfully closed."
	};
	
	[status setStringValue:statusTexts[inOperationCode]];
	[status displayIfNeeded];
	return EmaiErrNoErr;
}

EmaiError
ImapCallback::OnDataSent(
		Emai::ImapSession& inSession,
		EmaiUint32 inSentSize,
		EmaiUint32 inTotalSize
	)
{
	EmaiUint32 fieldSize = sizeof(NSTextField*);
	int percent = 0;
	int curPercent = (100 * inSentSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(ImapCurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		NSTextField* status = NULL;
		inSession.GetUserData(ImapPercentText, &status, fieldSize);
		[status setStringValue:[NSString stringWithFormat: @"%d %%", curPercent]];
		[status displayIfNeeded];
		inSession.SetUserData(ImapCurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

EmaiError
ImapCallback::OnDataReceived(
		Emai::ImapSession& inSession,
		EmaiUint32 inReceivedSize,
		EmaiUint32 inTotalSize
	)
{
	EmaiUint32 fieldSize = sizeof(NSTextField*);
	int percent = 0;
	int curPercent = (100 * inReceivedSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(ImapCurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		NSTextField* status = NULL;
		inSession.GetUserData(ImapPercentText, &status, fieldSize);
		[status setStringValue:[NSString stringWithFormat: @"%d %%", curPercent]];
		[status displayIfNeeded];
		inSession.SetUserData(ImapCurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

EmaiError
ImapCallback::OnServerError(
		Emai::ImapSession& inSession,
		EmaiConstCharPtr inServerReply,
		EmaiUint32 /*inReplyCode*/
	)
{
	EmaiUint32 statSize = sizeof(NSTextField*);
	NSTextField* status = NULL;
	NSString* text = [NSString stringWithFormat:@"The server error encountered was: %s", inServerReply];
	inSession.GetUserData(ImapStatusText, &status, statSize);
	[status setStringValue:text];
	[status displayIfNeeded];
	return EmaiErrNoErr;
}
