/*
// File:		ImapFolder.hpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#ifndef __IMAPFOLDER_HPP__
#define __IMAPFOLDER_HPP__

#include <EmaiIMAP.hpp>
#include <EmaiMessage.hpp>
#include <vector>

class ImapMessageData;

/*!
// @class	ImapFolder
// @brief	ImapFolder is a sample class which holds information about IMAP mailbox. It does not suport caching of the data between sessions.
//
 */
class ImapFolder
{
protected:
	EmaiUint32						mAttributes;
	Emai::StringList				mCustomAttributes;
	EmaiChar						mDelimiter;
	Emai::UniString					mFolderName;

	EmaiUint32						mFlags;
	Emai::StringList				mCustomFlags;
	bool							mReadOnly;
	
	EmaiUint32						mExists;
	EmaiUint32						mRecent;
	EmaiUint32						mUnseen;

	EmaiUint32						mPermanentFlags;
	Emai::StringList				mPermanentCustomFlags;
	
	EmaiUint32						mNextUid;
	EmaiUint32						mUidValilidy;
	Emai::ImapSequenceSet			mSelectedForDeletion;
	
	
	typedef std::vector<ImapMessageData*>
									MessageList;

	MessageList						mMessages;

	void							FreeMessageList();

	static bool						CompareData(
											const ImapMessageData* inData1, 
											const ImapMessageData* inData2);

protected:

	Emai::ImapSequenceSet			mFoundMessageList;


	virtual ImapMessageData*		CreateMessageData() const;

public:
									ImapFolder(
											const EmaiListResponse& inListResponse);
	virtual							~ImapFolder();

	const Emai::UniString&			GetName() const
									{
										return mFolderName;
									}
	void							SetName(
											const Emai::UniString& inName)
									{
										mFolderName = inName;
									}

	EmaiChar						GetDelimiter() const
									{
										return mDelimiter;
									}
	EmaiUint32						GetAttributes() const
									{
										return mAttributes;
									}
	const Emai::StringList&			GetCustomAttributes() const
									{
										return mCustomAttributes;
									}
	void							SetAttributes(
											EmaiUint32 inAttributes,
											const Emai::StringList& inCustomAttributes);

	EmaiUint32						GetFlags() const
									{
										return mFlags;
									}
	const Emai::StringList&			GetCustomFlags() const
									{
										return mCustomFlags;
									}
	void							SetFlags(
											EmaiUint32 inFlags,
											const Emai::StringList& inCustomFlags);

	void							SetExists(
											EmaiUint32 inExists);
	EmaiUint32						GetExists() const
									{
										return mExists;
									}
	void							SetRecent(
											EmaiUint32 inRecent);
	EmaiUint32						GetRecent() const
									{
										return mRecent;
									}
	void							SetUnseen(
											EmaiUint32 inUnseen);
	EmaiUint32						GetUnseen() const
									{
										return mUnseen;
									}

	void							SetPermanentFlags(
											EmaiUint32 inFlags,
											const Emai::StringList& inCustomFlags);

	void							SetReadOnly(
											bool inValue)
									{
										mReadOnly = inValue;
									}
	bool							IsReadOnly() const
									{
										return mReadOnly;
									}
	void							SetFoundMessageList(
											const Emai::ImapSequenceSet inNewFoundMessageList)
									{
										mFoundMessageList = inNewFoundMessageList;
									}
	const Emai::ImapSequenceSet&	GetFoundMessageList()
									{
										return mFoundMessageList;
									}

	void							SetSelectedForDeletion(
											const Emai::ImapSequenceSet& inSelectedForDeletion)
									{
										mSelectedForDeletion = inSelectedForDeletion;
									}
	Emai::ImapSequenceSet&			GetSelectedForDeletion()
									{
										return mSelectedForDeletion;
									}
	void							ResetSelectedForDeletion();

	bool							IsMarkedForDeletion(
										EmaiUint32 inMessageNumber);

	virtual void					SetNextUid(
											EmaiUint32 inUID);
	virtual void					SetUidValidity(
											EmaiUint32 inUIDValidity);

	virtual void					AddMessageData(
											const EmaiFetchResponse& inResponse);

	ImapMessageData*				GetMessageDataAtIndex(
											EmaiUint32 inMessageNumber);

	ImapMessageData*				GetMessageDataByUid(
											const EmaiUint32 inUID);

	virtual void					MessageDeleted(
											EmaiUint32 inMessageNumber);
	virtual void					MessagesAppended(
											EmaiUint32 inMessageCount,
											const EmaiMessage* inMessages,
											const Emai::ImapSequenceSet& inUidSequenceSet);
};


#endif /*__IMAPFOLDER_HPP__*/
