/*
// File:		Pop3GUIDlg.cpp
//
// Version:		mail.it library 1.0
//
// Copyright:	(C) 2007 Dacons LLP.
*/

#include PCH_FILE_NAME
#include "Pop3GUI.hpp"
#include "Pop3GUIDlg.hpp"

#include <EmaiPOP3.hpp>
#include "POP3Callback.hpp"
#include "ErrorsDescriptions.hpp"
#include <atlpath.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// Struct contains item data
struct ItemData
{
	Emai::UniString mFileName;
	bool mIsInline;
	EmaiUint32 mAttachmentIndex;
};

CPop3GUIDlg::CPop3GUIDlg(CWnd* pParent /*=NULL*/)
: CDialog(CPop3GUIDlg::IDD, pParent)
	, mMailServer(_T("your.mailserver.net"))
	, mAccount(_T("your.account"))
	, mPassword(_T("password"))
	, mFromWho(_T(""))
	, mToWho(_T(""))
	, mMessageText(_T(""))
	, mCC(_T(""))
	, mTextType(FALSE)
	, mRemoveMessages(FALSE)
	, mProgressPercent(_T(""))
	, mStatusText(_T(""))
	, mPort(EmaiDefaultPop3Port)
	, mUseSsl(FALSE)
	, mLicenseKey(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPop3GUIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_SERVER, mMailServer);
	DDX_Text(pDX, IDC_EDIT_ACCOUNT, mAccount);
	DDX_Text(pDX, IDC_EDIT_PASSWORD, mPassword);
	DDX_Text(pDX, IDC_STATUS_TEXT, mStatusText);
	DDX_Text(pDX, IDC_EDIT_FROM, mFromWho);
	DDX_Text(pDX, IDC_EDIT_TO, mToWho);
	DDX_Text(pDX, IDC_MESSAGE_TEXT, mMessageText);
	DDX_Text(pDX, IDC_EDIT_CC, mCC);
	DDX_Radio(pDX, IDC_RADIO_TEXT_TYPE_PLAIN, mTextType);
	DDX_Check(pDX, IDC_CHECK_REMOVE, mRemoveMessages);
	DDX_Control(pDX, IDC_COMBO_TIME, mTimePeriods);
	DDX_Text(pDX, IDC_PROGRESS_PERCENT, mProgressPercent);
	DDX_Text(pDX, IDC_EDIT_PORT, mPort);
	DDX_Check(pDX, IDC_CHECK_SSL, mUseSsl);
	DDX_Control(pDX, IDC_COMBO_AUTH_TYPE, mAuthType);
	DDX_Control(pDX, IDC_LIST_MESSAGES, mMessagesList);
	DDX_Control(pDX, IDC_LIST_ATTACHMENTS, mAttachmentsList);
	DDX_Text(pDX, IDC_EDIT_LICKEY, mLicenseKey);
}

BEGIN_MESSAGE_MAP(CPop3GUIDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_RECEIVE, OnBnClickedButtonReceive)
	ON_BN_CLICKED(IDC_RADIO_TEXT_TYPE_PLAIN, OnBnClickedRadioTextTypePlain)
	ON_BN_CLICKED(IDC_RADIO_TEXT_TYPE_HTML, OnBnClickedRadioTextTypeHtml)
	ON_BN_CLICKED(IDC_CHECK_REMOVE, OnBnClickedCheckRemove)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST_MESSAGES, OnLvnItemchangedListMessages)
	ON_BN_CLICKED(IDC_BUTTON_SAVE, OnBnClickedSave)
	ON_EN_KILLFOCUS(IDC_EDIT_LICKEY, OnEnKillfocusEditLickey)
	ON_EN_SETFOCUS(IDC_EDIT_LICKEY, OnEnSetfocusEditLickey)
END_MESSAGE_MAP()


BOOL CPop3GUIDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	ShowWindow(SW_NORMAL);

	mTimePeriods.SetCurSel(0);
	mAuthType.SetCurSel(0);

	mMessagesList.InsertColumn(0, _T("Subject"), LVCFMT_CENTER, -1, 0);
	mMessagesList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	mMessagesList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	mAttachmentsList.InsertColumn(0, _T("Attachments"), LVCFMT_CENTER, -1, 0);
	mAttachmentsList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	mAttachmentsList.SetExtendedStyle(LVS_EX_FULLROWSELECT);


	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPop3GUIDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

void CPop3GUIDlg::OnDestroy()
{
	RemoveAllAttachments();
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPop3GUIDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CPop3GUIDlg::FormatError(Emai::Exception& inException)
{
	AfxMessageBox(errorsDescriptions[::abs(inException.GetErrorCode())]);
}

void CPop3GUIDlg::OnBnClickedButtonReceive()
{
	UpdateData();
	if (EmaiRegisterLibrary(mLicenseKey) == EmaiErrNoErr)
	{
		CWaitCursor wait;
		EmaiUint32 currentMessageNumber = 0;
		try
		{
			Pop3Callback statusCallback;
			mMessagesList.DeleteAllItems();
			mFromWho = _T("");
			mToWho = _T("");
			mCC = _T("");
			mMessageText = _T("");
			mAttachmentsList.DeleteAllItems();
			mMessages.clear();

			EmaiOptions options = EmaiNullOptions;
			if (mUseSsl) 
				options = EmaiPop3UseSSLOption;
			Emai::Pop3Session pop3Session(statusCallback, 60, options);
			HWND hwnd = *GetDlgItem(IDC_PROGRESS_PERCENT);

			// Set handle of progress percent static text control as user data to change its in statusCallback
			int perc = 0;
			pop3Session.SetUserData(Pop3CurrentPercent, &perc, sizeof(EmaiUint32));
			pop3Session.SetUserData(Pop3StatusText, &hwnd, sizeof(HWND));
			hwnd = *GetDlgItem(IDC_STATUS_TEXT);
			pop3Session.SetUserData(Pop3PercentText, &hwnd, sizeof(HWND));
			pop3Session.SetUserData(Pop3CurrentMessage, &perc, sizeof(EmaiUint32));
			pop3Session.SetUserData(Pop3MessageNumber, &perc, sizeof(EmaiUint32));


			// Connect to pop3 server
			pop3Session.Connect(mMailServer, mPort, EmaiNullOptions);

			// Authenticate on server
			EmaiUint32 authTypes[] = {
				EmaiPOP3AuthAutomatic,
					EmaiPOP3AuthLoginCommand,
					EmaiPOP3AuthApop,
					EmaiPOP3AuthLogin,
					EmaiPOP3AuthPlain,
					EmaiPOP3AuthCramMD5,
					EmaiPOP3AuthNTLM
			};
			pop3Session.Authenticate(mAccount, mPassword, authTypes[mAuthType.GetCurSel()]);

			// Create history file path
			CPath path;
			if (!::SHGetSpecialFolderPath(m_hWnd, path.m_strPath.GetBufferSetLength(MAX_PATH), CSIDL_APPDATA, FALSE))
				path = AfxGetApp()->m_pszHelpFilePath;

			path.RemoveFileSpec();
			path.AddBackslash();
			Emai::MessageInfoList messageInfoList(path.m_strPath + CString(_T("net.dacons.pop3demo.messagehistory")));

			// Retrive message list from server
			pop3Session.RetrieveMessageList(messageInfoList);

			EmaiUint32 messageCount = messageInfoList.GetCount();
			pop3Session.SetUserData(Pop3MessageNumber, &messageCount, sizeof(EmaiUint32));
			CString msg;

			// Fill message list with messages
			while (currentMessageNumber < messageCount)
			{
				pop3Session.SetUserData(Pop3CurrentMessage, &currentMessageNumber, sizeof(EmaiUint32));
				EmaiMessageInfo outInfo;
				messageInfoList.GetInfo(currentMessageNumber, outInfo);

				Emai::Message message = pop3Session.RetrieveMessage(outInfo.messageNumber);
				mMessages.push_back(message);

				// Mark message received
				messageInfoList.MarkMessageReceived(++currentMessageNumber);
			}

			// Remove message from server if chose
			if (mRemoveMessages)
			{
				EmaiUint32 numDays[] = { 1, 7, 31 };

				messageInfoList.EvaluateSublist(numDays[mTimePeriods.GetCurSel()], EmaiMessagesForDeletion);

				currentMessageNumber = 0;
				messageCount = messageInfoList.GetCount(EmaiMessagesForDeletion);
				pop3Session.SetUserData(Pop3MessageNumber, &messageCount, sizeof(EmaiUint32));

				while (currentMessageNumber < messageCount)
				{
					pop3Session.SetUserData(Pop3CurrentMessage, &currentMessageNumber, sizeof(EmaiUint32));
					EmaiMessageInfo outInfo;
					messageInfoList.GetInfo(currentMessageNumber, outInfo, EmaiMessagesForDeletion);

					// Delete message with number outInfo.messageNumber
					pop3Session.DeleteMessage(outInfo.messageNumber);

					currentMessageNumber++;
				}
			}
			mProgressPercent = _T("");

			// Close session and disconnect from server
			pop3Session.Disconnect();
		}
		catch (Emai::Exception& exception)
		{
			FormatError(exception);
			mProgressPercent = _T("");
			UpdateData(FALSE);
		}

		if (mMessages.size() > 0)
		{
			CString subject;
			for (EmaiUint32 i = 0; i <= mMessages.size() - 1; i++)
			{
				try
				{
					Emai::Message message = mMessages.at(i);
					subject = message.GetField(NULL, EmaiSubject);
				}
				catch (Emai::Exception& /*exception*/)
				{
					subject = _T("Error while getting subject");
				}
				mMessagesList.InsertItem(mMessagesList.GetItemCount(), subject);
			}
			mMessagesList.SetSelectionMark(currentMessageNumber - 1);
			mMessagesList.SetItemState(currentMessageNumber - 1, LVIS_SELECTED, LVIS_SELECTED | LVIS_FOCUSED);
			mMessagesList.EnsureVisible(currentMessageNumber - 1, FALSE);
			mMessagesList.SetFocus();
		}
		mRemoveMessages = FALSE;
		mTimePeriods.EnableWindow(mRemoveMessages);
		UpdateData(FALSE);
		wait.Restore();
	}
	else
		AfxMessageBox(_T("Licence key is invalid. Restart application and enter valid key."));

}

CString CPop3GUIDlg::FormatTextForTextField(EmaiOptions inFieldType, 
								   MessagePartEnumerator& inEnumerator) const
{
	CString str = _T("");
	Emai::ContactList contacts = inEnumerator.GetContacts(inFieldType);
	EmaiUint32 count = contacts.GetCount();
	CString name;
	CString address;
	EmaiUint32 i = 0;
	for (i = 0; i < count; ++i)
	{
		name = contacts.GetName(i);
		address = contacts.GetAddress(i);
		if((name.GetLength() > 0) && (address.GetLength() > 0))
			str = str + name + _T(" - ") + address + _T("; ");
		else if((name.GetLength() > 0) && (address.GetLength() == 0)) str = str + name + _T("; ");
		else if((name.GetLength() == 0) && (address.GetLength() > 0))str = str + address + _T("; ");
	}
	return str;
}

void CPop3GUIDlg::OnBnClickedRadioTextTypePlain()
{
	UpdateData();
	if(mMessagesList.GetItemCount() > 0)
	{
		// Fill text field with plain text
		Emai::Message message = mMessages.at(mMessagesList.GetSelectionMark());
		MessagePartEnumerator messagePartEnumerator(message);
		messagePartEnumerator.Enumerate();
		Emai::UniString text;
		messagePartEnumerator.GetUnicodeText(text);
		mMessageText = CString(text.c_str());
		
		UpdateData(FALSE);
	}
}

void CPop3GUIDlg::OnBnClickedRadioTextTypeHtml()
{
	UpdateData();
	if(mMessagesList.GetItemCount() > 0)
	{
		// Fill text field with html text
		Emai::Message message = mMessages.at(mMessagesList.GetSelectionMark());
		MessagePartEnumerator messagePartEnumerator(message);
		messagePartEnumerator.Enumerate();
		Emai::String html;
		messagePartEnumerator.GetHtml(html);
		mMessageText = CString(html.c_str());
		UpdateData(FALSE);
	}
}
void CPop3GUIDlg::OnBnClickedCheckRemove()
{
	UpdateData();
	mTimePeriods.EnableWindow(mRemoveMessages);
}

void CPop3GUIDlg::OnLvnItemchangedListMessages(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
	if ((pNMLV->uOldState == 0) && (pNMLV->uNewState != 0))
	{
		try
		{
			Emai::Message message = mMessages.at(pNMLV->iItem);

			MessagePartEnumerator messagePartEnumerator(message);
			messagePartEnumerator.Enumerate();

			// Fill fields with data
			mFromWho = FormatTextForTextField(EmaiFrom, messagePartEnumerator);
			mToWho = FormatTextForTextField(EmaiTo, messagePartEnumerator);
			mCC = FormatTextForTextField(EmaiCC, messagePartEnumerator);

			// Fill attachment list with attachments
			RemoveAllAttachments();
			EmaiUint32 partsCount = messagePartEnumerator.GetAttachmentsCount();
			EmaiUint32 length = 0;
			CString attachmentInfo;
			EmaiUint32 i = 0;
			TCHAR buffer[256];
			ItemData* pData = NULL;
			int newIndex = 0;
			if (partsCount > 0)
			{
				Emai::UniString fileName;
				for (i = 0; i < partsCount; i++)
				{
					messagePartEnumerator.GetAttachmentAtIndex(i, fileName, NULL, length);
					::StrFormatByteSize(length, buffer, 256);
					attachmentInfo.Format(_T("%s (size = %s)"),fileName.c_str(), buffer);
					newIndex = mAttachmentsList.InsertItem(mAttachmentsList.GetItemCount(), attachmentInfo);
					pData = new ItemData;
					pData->mFileName = fileName;
					pData->mIsInline = false;
					pData->mAttachmentIndex = i;
					mAttachmentsList.SetItemData(newIndex, reinterpret_cast<DWORD_PTR>(pData));
				}
			}

			// Fill attachment list with inline attachments
			partsCount = messagePartEnumerator.GetInlinesCount();
			Emai::UniString contentId;
			if (partsCount > 0)
			{
				Emai::UniString fileName;
				for (i = 0; i < partsCount; i++)
				{
					messagePartEnumerator.GetInlineAtIndex(i, fileName, contentId, NULL, length);
					::StrFormatByteSize(length, buffer, 256);
					attachmentInfo.Format(_T("%s (size = %s content ID = %s)"),fileName.c_str(), buffer, contentId.c_str());
					newIndex = mAttachmentsList.InsertItem(mAttachmentsList.GetItemCount(), attachmentInfo);
					pData = new ItemData;
					pData->mFileName = fileName;
					pData->mIsInline = true;
					pData->mAttachmentIndex = i;
					mAttachmentsList.SetItemData(newIndex, reinterpret_cast<DWORD_PTR>(pData));
				}
			}

			Emai::UniString text;
			messagePartEnumerator.GetUnicodeText(text);
			Emai::String html;
			messagePartEnumerator.GetHtml(html);
			if (text.empty())
			{
				mTextType = TRUE;
				GetDlgItem(IDC_RADIO_TEXT_TYPE_PLAIN)->EnableWindow(FALSE);
				GetDlgItem(IDC_RADIO_TEXT_TYPE_HTML)->EnableWindow(FALSE);
				mMessageText = CString(html.c_str());
			}
			else if (html.empty())
			{
				mTextType = FALSE;
				GetDlgItem(IDC_RADIO_TEXT_TYPE_PLAIN)->EnableWindow(FALSE);
				GetDlgItem(IDC_RADIO_TEXT_TYPE_HTML)->EnableWindow(FALSE);
				mMessageText = CString(text.c_str());
			}
			else
			{
				GetDlgItem(IDC_RADIO_TEXT_TYPE_PLAIN)->EnableWindow(TRUE);
				GetDlgItem(IDC_RADIO_TEXT_TYPE_HTML)->EnableWindow(TRUE);
				if(!mTextType)
				{
					// Fill text field with plain text
					mMessageText = CString(text.c_str());
				}
				else
				{
					// Fill text field with html text
					mMessageText = CString(html.c_str());
				}
			}
		}
		catch (Emai::Exception& exception)
		{
			FormatError(exception);
		}
		UpdateData(FALSE);
	}
	*pResult = 0;
}

void CPop3GUIDlg::GetPath(EmaiConstUniCharPtr inDefaultFileName, Emai::UniString& outPath, Emai::UniString& outFileName)
{
	CPath folderPath = _T("");

	CFileDialog dlg(FALSE, NULL, inDefaultFileName);
	if (dlg.DoModal() == IDOK)
	{
		folderPath.m_strPath = dlg.GetPathName();
	}
	
	CPath fileName = folderPath;
	fileName.StripPath();
	folderPath.RemoveFileSpec();
	outPath = folderPath.m_strPath;
	outFileName = fileName.m_strPath;
}

void CPop3GUIDlg::OnBnClickedSave()
{
	if(mAttachmentsList.GetItemCount() > 0)
	{
		int mark = mAttachmentsList.GetSelectionMark();
		if (mark >= 0)
		{
			Emai::Message message = mMessages.at(mMessagesList.GetSelectionMark());
			MessagePartEnumerator messagePartEnumerator(message);
			messagePartEnumerator.Enumerate();
			if ((messagePartEnumerator.GetAttachmentsCount() > 0) || (messagePartEnumerator.GetInlinesCount() > 0))
			{
				ItemData* pData = (ItemData*)mAttachmentsList.GetItemData(mAttachmentsList.GetSelectionMark());
				Emai::UniString fileName;
				Emai::UniString filePath;
				GetPath(pData->mFileName.c_str(), filePath, fileName);
				if (!messagePartEnumerator.SaveAttachmentAtIndex(pData->mIsInline, pData->mAttachmentIndex, filePath.c_str(), fileName.c_str()))
					AfxMessageBox(_T("Can not save attachment."));
			}
		}
	}
}

void CPop3GUIDlg::RemoveAllAttachments()
{
	int count = mAttachmentsList.GetItemCount();
	if (count > 0)
	{
		for (int i = 0; i <= count - 1; i++)
			delete (ItemData*)mAttachmentsList.GetItemData(i);
	}
	mAttachmentsList.DeleteAllItems();
}

void CPop3GUIDlg::OnEnKillfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.IsEmpty())
	{
		mLicenseKey = _T("Put your license key here.");
		UpdateData(FALSE);
	}
}

void CPop3GUIDlg::OnEnSetfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.CompareNoCase(_T("Put your license key here.")) == 0)
	{
		mLicenseKey = _T("");
		UpdateData(FALSE);
	}
}
