/*!
// @file		EmaiErrors.h
//
// @brief		Result codes, used at the library
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAIERRORS_H__
#define __EMAIERRORS_H__

#include <EmaiTypes.h>

#ifdef __cplusplus
	extern "C" {
#endif

/*!
// @brief	Error codes
*/
typedef EmaiSint32						EmaiError;
enum EmaiEnumResultCodes
{
	/*! Operation successfully completed*/
	EmaiErrNoErr						= 0,

	/*! Cannot establish connection to the server*/
	EmaiErrConnectFailed				= -1,

	/*! Server didn't send conformation greeting to the client after connection*/
	/*! was established*/
	EmaiErrConnectUnconfirmed			= -2,
	
	/*! Connection was interrupted by user*/
	EmaiErrInterruptedByUser			= -5,
	
	/*! A network error occured*/
	EmaiErrConnectionReset				= -6,
	
	/*! Server rejected the command*/
	EmaiErrCommandFailed				= -7,
	
	/*! Server doesn't support the command*/
	EmaiErrCommandNotSupported			= -8,
	
	/*! Unexpected internal error*/
	EmaiErrUnknown						= -9,

	/*! Cannot switch into secured mode*/
	EmaiErrSslHandshakeFailed			= -28,

	/*! Function parameter is invalid*/
	EmaiErrIncorrectParameter			= -30,

	/*! Invalid registration data*/
	EmaiErrInvalidRegData				= -31,

	/*! The session is already connected to server*/
	EmaiErrAlreadyConnected				= -32,

	/*! The session is not connected to server*/
	EmaiErrNotConnected					= -33,

	/*! The field doesn't exist in message header*/
	EmaiErrNoSuchField					= -35,
	EmaiErrNoFromField					= EmaiErrNoSuchField,
	EmaiErrNoToField					= -36,

	/*! Connection is timed out*/
	EmaiErrTimeout						= -37,
	
	/*! Server address is invalid*/
	EmaiErrAddrInvalid					= -40,

	/*! The specified file cannot be found*/
	EmaiErrFileNotFound					= -42,
	
	/*! An error occured while reading or writing the specified file*/
	EmaiErrFileIO						= -43,

	/*! EmaiPop3MessageInfoListEvaluateSubList function should be called before*/
	/*! trying to work with the sublist*/
	EmaiErrSubListNotEvaluted			= -46,

	/*! Function which sends command on server was called from session callback*/
	EmaiErrCallNotInContext				= -50,

	/*! Unknown or unsupported charset is used in message, this error can appear*/
	/*! in two cases: character set is not supported on you OS (for instance, */
	/*! "Install files for East Asian laguages" check box in "Regional and Language*/
	/*! Options" control panel is turned off) or charset name is not recognized by*/
	/*! library at all. */
	/*! This error is returned by library only when EmaiFailOnUnknownCharset option*/
	/*! is specified for the corresponding function*/
	EmaiErrChartset						= -51
};

#ifdef __cplusplus
	}
#endif

#endif /*__EMAIERRORS_H__*/
